/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.utils.hibernate;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.hibernate.HibernateException;
import org.hibernate.usertype.UserType;

/**
 * 枚举用户类型
 *
 * @author Lucky Yang
 */
public abstract class EnumeratedUserType<E extends Enum<E>> implements UserType, Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7684322062228289127L;

	protected abstract Object toEnumObject(int value);
	protected abstract int fromEnumObject(Object value);
	protected abstract Class<E> getClazz();
	
	public int[] sqlTypes() {
		return new int[] { Types.SMALLINT };
	}

	@Override
	public Class<E> returnedClass() {
		return this.getClazz();
	}
	
	public boolean equals(Object arg0, Object arg1) throws HibernateException {
		return arg0 == null ? arg1 == null : arg0.equals(arg1);
	}

	public int hashCode(Object arg0) throws HibernateException {
		return arg0 == null ? 0 : arg0.hashCode();
	}

	public Object nullSafeGet(ResultSet rs, String[] names, Object owner) throws HibernateException, SQLException {
		int value = rs.getInt(names[0]);
		if (rs.wasNull())
			return null;
		else
			return this.toEnumObject(value);
	}

	public void nullSafeSet(PreparedStatement ps, Object value, int index) throws HibernateException, SQLException {
		if (value == null) 
			ps.setNull(index, Types.SMALLINT);
		else
			ps.setInt(index, this.fromEnumObject(value));
	}

	public Object deepCopy(Object value) throws HibernateException {
		return value;
	}

	public boolean isMutable() {
		return false;
	}

	public Serializable disassemble(Object value) throws HibernateException {
		return (Serializable) value;
	}

	public Object assemble(Serializable cached, Object owner) throws HibernateException {
		return cached;
	}

	public Object replace(Object original, Object target, Object owner) throws HibernateException {
		return original;
	}

}
