/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.utils.hibernate;

import java.sql.SQLException;
import java.util.Collection;
import java.util.List;
import java.util.Map.Entry;

import net.sourceforge.kivu4j.utils.api.repository.FilterParameter;
import net.sourceforge.kivu4j.utils.api.repository.ParameterException;
import net.sourceforge.kivu4j.utils.lang.domain.Filter;
import net.sourceforge.kivu4j.utils.lang.domain.Filter.COMPARISON;
import net.sourceforge.kivu4j.utils.lang.domain.Filter.DIR;

import org.apache.commons.lang.StringUtils;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.orm.hibernate3.HibernateCallback;

/**
 * Filter查询回调类
 *
 * @author Lucky Yang
 */
public class FilterParameterExecutorCallback<T> 
	implements HibernateCallback<Collection<T>> {
	
	private FilterParameter parameter;
	private Class<T> clazz;
	
	public FilterParameterExecutorCallback(Class<T> clazz, FilterParameter parameter) {
		super();
		this.clazz = clazz;
		this.parameter = parameter;
	}
    
	@SuppressWarnings("unchecked")
	@Override
	public Collection<T> doInHibernate(Session session) 
		throws HibernateException, SQLException {
		
		Criteria main = session.createCriteria(this.clazz);
		for (Entry<String, List<Filter>> e : this.parameter.getParameters().entrySet()) {
			if (FilterParameter.DEFAULT_ALIAS.equals(e.getKey())){
				this.buildCriteria(main, e.getValue());
			}else{
				Criteria item = main.createCriteria(e.getKey());
				this.buildCriteria(item, e.getValue());
			}
		}		
		
        if (this.parameter.isUnique()){
        	main.setMaxResults(1);	
        }else if (this.parameter.getLimit() > 0){
        	main.setFirstResult(this.parameter.getStart());
        	main.setMaxResults(this.parameter.getLimit() + 1);        	
        }
        
        if (this.parameter.isDistanct())
        	main.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
        return main.list();  
	}

	private void addOrder(Criteria criteria, Filter filter){
		if (StringUtils.isBlank(filter.getDir())) return;
		
		try{
			switch (DIR.valueOf(filter.getDir().toUpperCase())){
				case ASC:
					criteria.addOrder(Order.asc(filter.getRealColumnName()));
					break;
				case DESC:
					criteria.addOrder(Order.desc(filter.getRealColumnName()));
					break;					
			}
		}catch(IllegalArgumentException e){
			throw new ParameterException(filter.getDir() + " not supported");
		}			
	}
	
	private void addRestrictions(Criteria criteria, Filter filter){
		if (StringUtils.isBlank(filter.getComparison())) return;
		
		try{
			switch (COMPARISON.valueOf(filter.getComparison().toUpperCase())){				
				case EQ:
					criteria.add(Restrictions.eq(filter.getRealColumnName(), filter.getRealValue()));
					break;	
				case LT:
					criteria.add(Restrictions.lt(filter.getRealColumnName(), filter.getRealValue()));
					break;	
				case GT:
					criteria.add(Restrictions.gt(filter.getRealColumnName(), filter.getRealValue()));
					break;
				case IN:
					criteria.add(Restrictions.in(filter.getRealColumnName(), (Object[])filter.getRealValue()));
					break;
				case LIKE:
					criteria.add(Restrictions.like(filter.getRealColumnName(), filter.getRealValue()));
					break;					
			}		
		}catch(IllegalArgumentException e){
			throw new ParameterException(filter.getComparison() + " not supported");
		}
		
	}
	
	private void buildCriteria(Criteria criteria, List<Filter> filters){
		for(Filter filter:filters){
			this.addRestrictions(criteria, filter);
			this.addOrder(criteria, filter);
		}
	}		
}
