/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.utils.hibernate.support;

import org.apache.commons.lang.StringUtils;
import org.hibernate.cfg.DefaultNamingStrategy;


/**
 * 命名策略。如：
 * 
 * tableName -> table_name
 * columnName -> column_name
 * 
 *
 * @author Lucky Yang
 * @see org.hibernate.cfg.DefaultNamingStrategy
 */
public class DatabaseNamingStrategy extends DefaultNamingStrategy {

	private static final long serialVersionUID = 4614294361681654588L;
	
	private String tablePrefix;
	private Boolean isAddUnderscores;
	private Integer maxLength;
	
	protected String addUnderscores(String name) {
		if (name == null || ! this.isAddUnderscores) return name;
		
		StringBuffer stringBuffer = new StringBuffer(name.replace('.', '_'));
		for (int i = 1; i < stringBuffer.length() - 1; i++) {
			if (Character.isLowerCase(stringBuffer.charAt(i-1)) && Character.isUpperCase(stringBuffer.charAt(i)) && Character.isLowerCase(stringBuffer.charAt(i+1))) {
				stringBuffer.insert(i++, '_');
			}
		}
		return stringBuffer.toString().toLowerCase();
	}
	
	@Override
	public String classToTableName(String className) {
		String value = super.classToTableName(className);
		
		return StringUtils.substring(tablePrefix + addUnderscores(value), 0, maxLength);
	}

	@Override
	public String collectionTableName(String ownerEntity, String ownerEntityTable, String associatedEntity, String associatedEntityTable, String propertyName) {
		String value = super.collectionTableName(ownerEntity, ownerEntityTable, associatedEntity, associatedEntityTable, propertyName);
		
		return StringUtils.substring(tablePrefix + addUnderscores(value), 0, maxLength);
	}

	@Override
	public String logicalCollectionTableName(String tableName, String ownerEntityTable, String associatedEntityTable, String propertyName) {
		String value = super.logicalCollectionTableName(tableName, ownerEntityTable, associatedEntityTable, propertyName);
		
		return StringUtils.substring(tablePrefix + addUnderscores(value), 0, maxLength);
	}
	
	@Override
	public String propertyToColumnName(String propertyName) {
		String value = super.propertyToColumnName(propertyName);
		
		return StringUtils.substring(tablePrefix + addUnderscores(value), 0, maxLength);
	}

	@Override
	public String foreignKeyColumnName(String propertyName, String propertyEntityName, String propertyTableName, String referencedColumnName) {
		String value = super.foreignKeyColumnName(propertyName, propertyEntityName, propertyTableName, referencedColumnName);
		
		return StringUtils.substring(tablePrefix + addUnderscores(value), 0, maxLength);
	}

	@Override
	public String logicalCollectionColumnName(String columnName, String propertyName, String referencedColumn) {
		String value = super.logicalCollectionColumnName(columnName, propertyName, referencedColumn);
		
		return StringUtils.substring(tablePrefix + addUnderscores(value), 0, maxLength);
	}

	@Override
	public String logicalColumnName(String columnName, String propertyName) {
		String value = super.logicalColumnName(columnName, propertyName);
		
		return StringUtils.substring(tablePrefix + addUnderscores(value), 0, maxLength);
	}

	@Override
	public String joinKeyColumnName(String joinedColumn, String joinedTable) {
		String value = super.joinKeyColumnName(joinedColumn, joinedTable);
		
		return StringUtils.substring(tablePrefix + addUnderscores(value), 0, maxLength);
	}

	public String getTablePrefix() {
		return tablePrefix;
	}

	public void setTablePrefix(String tablePrefix) {
		this.tablePrefix = tablePrefix;
	}

	public Boolean getIsAddUnderscores() {
		return isAddUnderscores;
	}

	public void setIsAddUnderscores(Boolean isAddUnderscores) {
		this.isAddUnderscores = isAddUnderscores;
	}

	public Integer getMaxLength() {
		return maxLength;
	}

	public void setMaxLength(Integer maxLength) {
		this.maxLength = maxLength;
	}

}